require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.database.sqlite.SQLiteDatabase"
import "android.graphics.Typeface"
import "android.os.Vibrator"

dir=package.searchpath("main",package.path):match("(.*)main.lua")

function initDB()
  local db = SQLiteDatabase.openOrCreateDatabase(dir.."/tasbih.db", nil, nil)
  db.execSQL([[CREATE TABLE IF NOT EXISTS tasbih_counter (id INTEGER PRIMARY KEY AUTOINCREMENT,count INTEGER DEFAULT 0,target INTEGER DEFAULT 0);]])
  cursor = db.rawQuery("SELECT * FROM tasbih_counter LIMIT 1", nil)
  if not cursor.moveToFirst() then
    db.execSQL("INSERT INTO tasbih_counter (count, target) VALUES (0, 0)")
  end
  cursor.close()
  return db
end

currentCount = 0
currentTarget = 0
vibrator = service.getSystemService(Context.VIBRATOR_SERVICE)

function createMainLayout()
  layout = {
    LinearLayout;
    orientation="vertical";
    layout_width="match_parent";
    layout_height="match_parent";
    backgroundColor="#FFFFFF";
    {
      LinearLayout;
      orientation="horizontal";
      layout_width="match_parent";
      layout_height="wrap_content";
      gravity="center";
      {
        Button;
        id="exitBtn";
        text="Keluar";
        textSize="14sp";
        textColor="#666666";
        layout_width="wrap_content";
        layout_height="wrap_content";
        layout_margin="8dp";
      };
    };
    {
      TextView;
      id="counterText";
      text="0";
      textSize="120sp";
      textColor="#000000";
      gravity="center";
      typeface=Typeface.DEFAULT_BOLD;
      layout_width="match_parent";
      layout_height="wrap_content";
      layout_weight="1";
    };
    {
      TextView;
      id="targetText";
      text="Target: 0";
      textSize="24sp";
      textColor="#666666";
      gravity="center";
      layout_width="match_parent";
      layout_height="wrap_content";
      layout_marginBottom="16dp";
    };
    {
      LinearLayout;
      orientation="vertical";
      layout_width="match_parent";
      layout_height="wrap_content";
      layout_marginBottom="32dp";
      gravity="center";
      {
        Button;
        id="plusBtn";
        text="Tambah Hitungan";
        textSize="16sp";
        textColor="#FFFFFF";
        backgroundColor="#4CAF50";
        layout_width="match_parent";
        layout_height="wrap_content";
        layout_margin="16dp";
        layout_marginLeft="32dp";
        layout_marginRight="32dp";
      };
      {
        Button;
        id="minusBtn";
        text="Kurangi Hitungan";
        textSize="16sp";
        textColor="#FFFFFF";
        backgroundColor="#4CAF50";
        layout_width="match_parent";
        layout_height="wrap_content";
        layout_margin="16dp";
        layout_marginLeft="32dp";
        layout_marginRight="32dp";
      };
    };
    {
      LinearLayout;
      orientation="vertical";
      layout_width="match_parent";
      layout_height="wrap_content";
      gravity="center";
      {
        Button;
        id="targetBtn";
        text="Atur Target Hitungan";
        textColor="#FFFFFF";
        backgroundColor="#2196F3";
        layout_width="match_parent";
        layout_height="wrap_content";
        layout_margin="8dp";
        layout_marginLeft="32dp";
        layout_marginRight="32dp";
      };
      {
        Button;
        id="resetBtn";
        text="Reset";
        textColor="#FFFFFF";
        backgroundColor="#F44336";
        layout_width="match_parent";
        layout_height="wrap_content";
        layout_margin="8dp";
        layout_marginLeft="32dp";
        layout_marginRight="32dp";
      };
    };
    {
      Button;
      id="helpBtn";
      text="Petunjuk";
      textColor="#666666";
      backgroundColor="#EEEEEE";
      layout_width="match_parent";
      layout_height="wrap_content";
      layout_margin="16dp";
      layout_marginLeft="32dp";
      layout_marginRight="32dp";
    };
  }

  mainDialog = LuaDialog(service)
  mainDialog.setView(loadlayout(layout))
  service.speak("Selamat datang")
  os.execute("sleep 1")
  mainDialog.show()
end

function loadCounter()
  local db = initDB()
  cursor = db.rawQuery("SELECT count, target FROM tasbih_counter LIMIT 1", nil)
  if cursor.moveToFirst() then
    currentCount = cursor.getInt(0)
    currentTarget = cursor.getInt(1)
    counterText.Text = tostring(currentCount)
    targetText.Text = "Target: " .. tostring(currentTarget)
  end
  cursor.close()
  db.close()
end

function saveCounter(count, target)
  local db = initDB()
  db.execSQL("UPDATE tasbih_counter SET count = ?, target = ? WHERE id = 1", {count, target})
  db.close()
end

function vibrate(duration)
  if vibrator then
    vibrator.vibrate(duration)
  end
end

function showTargetDialog()
  dlg = LuaDialog(service)
  dlg.setTitle("Atur Target")
  dlg.setView(loadlayout({
    LinearLayout;
    orientation="vertical";
    padding="16dp";
    {
      EditText;
      id="targetInput";
      hint="Masukkan target hitungan";
      inputType="number";
      layout_width="match_parent";
      layout_height="wrap_content";
    };
  }))
  dlg.setPositiveButton("OK",function()
    local newTarget = tonumber(targetInput.Text)
    if newTarget and newTarget > 0 then
      currentTarget = newTarget
      targetText.Text = "Target: "..newTarget
      saveCounter(currentCount, newTarget)
      vibrate(100)
    else
      Toast.makeText(service, "Masukkan angka yang valid", Toast.LENGTH_SHORT).show()
    end
  end)
  dlg.setNegativeButton("Batal",nil)
  dlg.show()
end

function onCreate()
  createMainLayout()
  loadCounter()
  
  exitBtn.onClick = function()
    service.playSoundClock()
    this.asyncSpeak("Terima kasih telah menggunakan extensi tasbih digital, salam sehat dan berkah selalu...")
    mainDialog.dismiss()
  end
  
  plusBtn.onClick = function()
    currentCount = currentCount + 1
    counterText.Text = tostring(currentCount)
    saveCounter(currentCount, currentTarget)
    vibrate(50)
    
    if currentTarget > 0 and currentCount >= currentTarget then
      vibrate(500)
      service.speak("Target tercapai")
      dlg = LuaDialog(service)
      dlg.setTitle("Target Tercapai!")
      dlg.setMessage("Anda telah mencapai target "..currentTarget.." hitungan")
      dlg.setPositiveButton("OK",nil)
      dlg.show()
    end
  end
  
  minusBtn.onClick = function()
    if currentCount > 0 then
      currentCount = currentCount - 1
      counterText.Text = tostring(currentCount)
      saveCounter(currentCount, currentTarget)
      vibrate(50)
    end
  end
  
  targetBtn.onClick = function()
    showTargetDialog()
  end
  
  resetBtn.onClick = function()
    dlg = LuaDialog(service)
    dlg.setTitle("Reset Hitungan")
    dlg.setMessage("Apakah Anda yakin ingin mereset hitungan?")
    dlg.setPositiveButton("Ya",function()
      currentCount = 0
      currentTarget = 0
      counterText.Text = "0"
      targetText.Text = "Target: 0"
      saveCounter(0, 0)
      vibrate(200)
    end)
    dlg.setNegativeButton("Tidak",nil)
    dlg.show()
  end
  
  helpBtn.onClick = function()
    dofile(dir .. "help.lua")
  end
end

onCreate()